
/*************************************************
****        VEGA - x86 CPU utilities          ****
**** Copyright 1996-2002, Alessandro Pedretti ****
*************************************************/


#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef __BORLANDC__
#  pragma hdrstop
#  pragma inline
#endif

#include "dxwcpu.h"


/**** Constructor ****/

DXWCPU::DXWCPU(void)
{
  char          *Tmp;
  uint32        k, j;
  SYSTEM_INFO   SysInfo;

  m_CPU_Features       = 0;
  m_CPU_GotInfo        = false;
  m_MaxCPUIDVal        = 0;
  m_ProcBrandID        = 0;
  m_ProcType           = 0;
  m_ProcFamily         = 0;
  m_ProcExtendedFamily = 0;
  m_ProcModel          = 0;
  m_ProcExtendedModel  = 0;
  m_ProcStepping       = 0;

  /**** Get the number of CPUs ****/

  ZeroMemory(&SysInfo, sizeof(SYSTEM_INFO));
  GetSystemInfo(&SysInfo);
  m_CPU_Number = SysInfo.dwNumberOfProcessors;

  for(k = 0; k < CPU_SUFF_MAXNUM; ++k) m_CPU_Suffix[k] = NULL;
  ZeroMemory(m_CPU_Suffix, sizeof(char *) * CPU_SUFF_MAXNUM);
  m_CPU_Suffix[0]      = CPU_SUFF_INTEL_586;
  m_CPU_Suffixes       = 1;

  GetInfo();

  for(k = 0, j = m_CPU_Suffixes - 1; k < j; ++k, --j) {
    Tmp             = m_CPU_Suffix[k];
    m_CPU_Suffix[k] = m_CPU_Suffix[j];
    m_CPU_Suffix[j] = Tmp;
  }
}


/**** Find a file ****/

bool DXWCPU::FindFile(char *FileName)
{
  char          *Prefix, **Suffix;

  FILE          *FH = NULL;

  Prefix = FileName + strlen(FileName) - 4;
  for(Suffix = m_CPU_Suffix; (FH == NULL)&& (*Suffix); ++Suffix) {
    *Prefix = 0;
    strcat(FileName, *Suffix);
    strcat(FileName, ".exe");
    if ((FH = fopen(FileName, "r")) != NULL) fclose(FH);
  } /* End of for */

  return (FH != NULL);
}


/**** Execute CPUID function EAX=funcNum and get EAX result ****/

uint32 DXWCPU::GetCPUID_EAX(uint32 funcNum)
{
  uint32        retval;

#ifdef __GNUC__
  try {
    __asm__("cpuid"
            :"=a" (retval)
            :"0" (funcNum)
            :"bx", "cx", "dx");
  } catch (...) {
    retval = 0;
  }

#else
  __try {
    _asm {
      mov eax, funcNum
      CPUID
      mov retval, eax
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    retval = 0;
  }
#endif

  return retval;
}


/**** Execute CPUID function EAX=funcNum and get EDX result ****/

uint32 DXWCPU::GetCPUID_EDX(uint32 funcNum)
{
  uint32        retval;

#ifdef __GNUC__
  uint32        EaxReg;

  try {
    __asm__("cpuid"
            : "=a" (EaxReg), "=d" (retval)
            : "0" (funcNum)
            : "bx", "cx");
  } catch (...) {
    retval = 0;
  }
#else
  __try {
    _asm {
      mov eax, funcNum
      CPUID
      mov retval, edx
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    retval = 0;
  }
#endif

  return retval;
}


/**** Execute CPUID EAX=funcNum and get EAX result and ****/
/**** the string EBX:EDX:ECX                           ****/

uint32 DXWCPU::GetCPUIDString(uint32 funcNum, char *String)
{
  uint32    retval;

#ifdef __GNUC__
  uint32    EbxReg, EcxReg, EdxReg;
  try {
    __asm__("cpuid"
            : "=a" (retval),
              "=b" (EbxReg),
              "=c" (EcxReg),
              "=d" (EdxReg)
            : "0" (funcNum));

    ((uint32 *)String)[0] = EbxReg;
    ((uint32 *)String)[1] = EdxReg;
    ((uint32 *)String)[2] = EcxReg;
  } catch (...) {
    retval = 0;
  }
#else
  __try {
    _asm {
      mov	eax,funcNum
      CPUID
      mov	retval,eax
      mov	eax,String
      mov	dword ptr[eax],ebx
      mov	dword ptr[eax+4],edx
      mov	dword ptr[eax+8],ecx
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    retval = 0;
  }
#endif

  return retval;
}


/**** AMD Specific ****/

void DXWCPU::GetCPUIDStringAMD(uint32 funcNum, char *String)
{
  uint32	retval;

#ifdef __GNUC__
  uint32    EbxReg, EcxReg, EdxReg;
  try {
    __asm__("cpuid"
            : "=a" (retval),
              "=b" (EbxReg),
              "=c" (EcxReg),
              "=d" (EdxReg)
            : "0" (funcNum));

    ((uint32 *)String)[0] = retval;
    ((uint32 *)String)[1] = EbxReg;
    ((uint32 *)String)[2] = EcxReg;
    ((uint32 *)String)[3] = EdxReg;
      } catch (...) {
    retval = 0;
  }
#else
  __try {
    _asm {
      mov       eax,funcNum
      CPUID
      mov	retval,eax
      mov	eax,String
      mov	dword ptr[eax+4],ebx
      mov	dword ptr[eax+8],ecx
      mov	ebx,retval
      mov	dword ptr[eax+12],edx
      mov	dword ptr[eax],ebx
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    retval = 0;
  }
#endif
}


/**** Get the information about the CPU ****/

bool DXWCPU::GetInfo(void)
{
  uint32        EaxReg, EbxReg, EdxReg;

  if (m_CPU_GotInfo) return true;
  m_CPU_GotInfo = true;

  ZeroMemory(m_ProcVendorString, sizeof(m_ProcVendorString));
  m_MaxCPUIDVal	= GetCPUIDString(0, m_ProcVendorString);


  /**** CPUID 1 ****/

#ifdef __GNUC__
  try {
    __asm__("cpuid"
            : "=a" (EaxReg),
              "=b" (EbxReg),
              "=d" (EdxReg)
            : "0" (1)
            : "cx");

  } catch (...) {
    return false;
  }
#else
  __try {
    _asm {
      mov       eax, 1
      cpuid
      mov       EaxReg, eax
      mov       EbxReg, ebx
      mov       EdxReg, edx
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    return false;
  }
#endif

  m_ProcType     = (EaxReg >> 12) & 0x3;
  m_ProcFamily   = (EaxReg >> 8 ) & 0xf;
  m_ProcModel    = (EaxReg >> 4 ) & 0xf;
  m_ProcStepping = (EaxReg)       & 0x7;
  m_CPU_Features = EdxReg;

  /**** Intel CPU ****/

  if ((strncmp(m_ProcVendorString, VENDOR_INTEL, 12)) == 0) {
    m_CPU_Type           = EaxReg;
    m_ProcBrandID        = EbxReg & 0xF;
    strcpy(m_ProcName, GetIntelProcName());

    if (m_ProcFamily >= 6) {
      m_CPU_Suffix[m_CPU_Suffixes++] = CPU_SUFF_INTEL_686;
      if (m_ProcModel >= 7)
        m_CPU_Suffix[m_CPU_Suffixes++] = CPU_SUFF_INTEL_P3;
    }
    if (m_ProcFamily >= 15) m_CPU_Suffix[m_CPU_Suffixes++] = CPU_SUFF_INTEL_P4;

    /**** AMD CPU ****/

  } else if ((strncmp(m_ProcVendorString, VENDOR_AMD, 12))==0) {
    if (GetCPUID_EAX(0x80000000)) {
      m_CPU_ExtendedFeatures = GetCPUID_EDX(0x80000001);
      GetCPUIDStringAMD(0x80000002, m_ProcName);
      GetCPUIDStringAMD(0x80000003, m_ProcName + 16);
      GetCPUIDStringAMD(0x80000004, m_ProcName + 32);
    } else m_CPU_ExtendedFeatures = EdxReg;

    if (m_ProcFamily >= 6)
      m_CPU_Suffix[m_CPU_Suffixes++] = CPU_SUFF_INTEL_686;
    if (m_ProcFamily >= 5)
      m_CPU_Suffix[m_CPU_Suffixes++] = CPU_SUFF_AMD_K6;
    if (m_ProcFamily >= 6) {
      m_CPU_Suffix[m_CPU_Suffixes++] = CPU_SUFF_AMD_K7;
      if (m_CPU_Features & INTEL_SSE_FLAG)
        m_CPU_Suffix[m_CPU_Suffixes++] = CPU_SUFF_AMD_K7SSE;
    }
    if (m_ProcFamily >= 15)
      m_CPU_Suffix[m_CPU_Suffixes++] = CPU_SUFF_AMD_A64;
  }

  return true;
}


/**** Obtain the Intel's processor name ****/

char *DXWCPU::GetIntelProcName(void)
{
  char          *CPUName = PROCESSOR_UNKNOWN;

  switch (m_ProcFamily) {
/*
  case 4:       // 486 family
    if (Model == 4) CPUName = INTEL_486SL;
    if (Model == 7) CPUName = INTEL_DX2WB;
    if (Model == 8) CPUName = INTEL_DX4;
    if ((Model == 8)&& (Type == 1)) CPUName = INTEL_DX4O;
    break;
*/

  case 5:       // 586 family
    if (m_ProcType == 1) CPUName = INTEL_PO;
    else CPUName = INTEL_P;
    break;

  case 6:       // 686 family
    switch(m_ProcModel) {
    case 1:
      CPUName = INTEL_PPRO;
      break;

    case 3:
      if (m_ProcType == 1) CPUName = INTEL_PIIO;
      else CPUName = INTEL_PII;
      break;

    case 5:
      CPUName = INTEL_PII;
      break;

    case 6:
      CPUName = INTEL_CELERON;
      break;

    case 7:
    case 8:
      CPUName = INTEL_PIII;
      break;

    case 10:
      CPUName = INTEL_XEON;
      break;

    case 0x0B:
      CPUName = INTEL_PIIIM;
      break;
    }
    break;

  case 15:      // Pentium 4 family
    m_ProcExtendedFamily = (m_CPU_Type >> 20) & 0xff;
    switch(m_ProcExtendedFamily) {
    case 0:     // Family = 15, Ext. Family = 0:  Pentium 4 (80786 ??) processor family
      m_ProcExtendedModel = (m_CPU_Type >> 16) & 0xFF;
      switch(m_ProcModel) {
      case 0:
      case 1:
        if (m_ProcBrandID == 8) CPUName = INTEL_PIV;
        else CPUName = INTEL_PIVXEON;
        break;

      case 2:
        if ((m_ProcBrandID == 9) || (m_ProcBrandID == 0xA)) CPUName = INTEL_PIV;
        else CPUName = INTEL_PIVXEON;
        break;

      default:
        CPUName = INTEL_PIV;
      } /* End of switch */
      break;

    case 1:     // Family = 15, Ext. Family = 1:  McKinley (64-bit) processor family
      CPUName = INTEL_MCKINLEY;
      break;

    default:
      CPUName = "Unknown Pentium 4+";
      break;
   } /* End of switch */
    break;
  } /* End of switch */

  return CPUName;
}


/**** Load library ****/

HINSTANCE DXWCPU::LoadLibrary(LPCTSTR lpszLibrary)
{
  char          Buf[MAX_PATH];
  char          *Prefix, **Suffix;

  HINSTANCE     hLib = NULL;

  strcpy(Buf, lpszLibrary);
  Prefix = Buf + strlen(Buf) - 4;
  for(Suffix = m_CPU_Suffix; (hLib == NULL)&& (*Suffix); ++Suffix) {
    *Prefix = 0;
    strcat(Buf, *Suffix);
    strcat(Buf, ".dll");
    hLib = ::LoadLibrary(Buf);
  } /* End of for */

  return hLib;
}

