
/*************************************************
****        VEGA - x86 CPU utilities          ****
**** Copyright 1996-2003, Alessandro Pedretti ****
*************************************************/


#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef __BORLANDC__
#  pragma hdrstop
#  pragma inline
#endif

#ifndef DXWCPU_C
#  define DXWCPU_C
#endif

#include "dxwcpu.h"



/**** Find a file ****/

uint32 DXWCPU_FindFile(DXWCPU *Cpu, char *FileName)
{
  char          *Prefix, **Suffix;

  FILE          *FH = NULL;

  Prefix = FileName + strlen(FileName) - 4;
  for(Suffix = Cpu -> m_CPU_Suffix; (FH == NULL)&& (*Suffix); ++Suffix) {
    *Prefix = 0;
    strcat(FileName, *Suffix);
    strcat(FileName, ".exe");
    if ((FH = fopen(FileName, "r")) != NULL) fclose(FH);
  } /* End of for */

  return (FH != NULL);
}


/**** Execute CPUID function EAX=funcNum and get EAX result ****/

uint32 DXWCPU_GetCPUID_EAX(uint32 funcNum)
{
  uint32        retval;

#ifdef __GNUC__
  __asm__("cpuid"
          :"=a" (retval)
          :"0" (funcNum)
          :"bx", "cx", "dx");

#else
  __try {
    _asm {
      mov eax, funcNum
      CPUID
      mov retval, eax
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    retval = 0;
  }
#endif

  return retval;
}


/**** Execute CPUID function EAX=funcNum and get EDX result ****/

uint32 DXWCPU_GetCPUID_EDX(uint32 funcNum)
{
  uint32        retval;

#ifdef __GNUC__
  uint32        EaxReg;

  __asm__("cpuid"
          : "=a" (EaxReg), "=d" (retval)
          : "0" (funcNum)
          : "bx", "cx");
#else
  __try {
    _asm {
      mov eax, funcNum
      CPUID
      mov retval, edx
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    retval = 0;
  }
#endif

  return retval;
}


/**** Execute CPUID EAX=funcNum and get EAX result and ****/
/**** the string EBX:EDX:ECX                           ****/

uint32 DXWCPU_GetCPUIDString(uint32 funcNum, char *String)
{
  uint32    retval;

#ifdef __GNUC__
  uint32    EbxReg, EcxReg, EdxReg;
  __asm__("cpuid"
          : "=a" (retval),
            "=b" (EbxReg),
            "=c" (EcxReg),
            "=d" (EdxReg)
          : "0" (funcNum));

    ((uint32 *)String)[0] = EbxReg;
    ((uint32 *)String)[1] = EdxReg;
    ((uint32 *)String)[2] = EcxReg;
#else
  __try {
    _asm {
      mov	eax,funcNum
      CPUID
      mov	retval,eax
      mov	eax,String
      mov	dword ptr[eax],ebx
      mov	dword ptr[eax+4],edx
      mov	dword ptr[eax+8],ecx
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    retval = 0;
  }
#endif

  return retval;
}


/**** AMD Specific ****/

void DXWCPU_GetCPUIDStringAMD(uint32 funcNum, char *String)
{
  uint32	retval;

#ifdef __GNUC__
  uint32    EbxReg, EcxReg, EdxReg;

  __asm__("cpuid"
          : "=a" (retval),
            "=b" (EbxReg),
            "=c" (EcxReg),
            "=d" (EdxReg)
          : "0" (funcNum));

  ((uint32 *)String)[0] = retval;
  ((uint32 *)String)[1] = EbxReg;
  ((uint32 *)String)[2] = EcxReg;
  ((uint32 *)String)[3] = EdxReg;

#else
  __try {
    _asm {
      mov       eax,funcNum
      CPUID
      mov	retval,eax
      mov	eax,String
      mov	dword ptr[eax+4],ebx
      mov	dword ptr[eax+8],ecx
      mov	ebx,retval
      mov	dword ptr[eax+12],edx
      mov	dword ptr[eax],ebx
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    retval = 0;
  }
#endif
}


/**** Get the information about the CPU ****/

uint32 DXWCPU_GetInfo(DXWCPU *Cpu)
{
  char          *Tmp;
  uint32        k, j;
  SYSTEM_INFO   SysInfo;
  uint32        EaxReg, EbxReg, EdxReg;


  ZeroMemory(Cpu, sizeof(DXWCPU));


  /**** Get the number of CPUs ****/

  ZeroMemory(&SysInfo, sizeof(SYSTEM_INFO));
  GetSystemInfo(&SysInfo);
  Cpu -> m_CPU_Number        = SysInfo.dwNumberOfProcessors;
  Cpu -> m_CPU_Suffix[0]     = CPU_SUFF_INTEL_586;
  Cpu -> m_CPU_Suffixes      = 1;

  Cpu -> m_MaxCPUIDVal	= DXWCPU_GetCPUIDString(0, Cpu -> m_ProcVendorString);

  /**** CPUID 1 ****/

#ifdef __GNUC__
  __asm__("cpuid"
          : "=a" (EaxReg),
            "=b" (EbxReg),
            "=d" (EdxReg)
          : "0" (1)
          : "cx");

#else
  __try {
    _asm {
      mov       eax, 1
      cpuid
      mov       EaxReg, eax
      mov       EbxReg, ebx
      mov       EdxReg, edx
    }
  } __except(EXCEPTION_EXECUTE_HANDLER) {
    return false;
  }
#endif

  Cpu -> m_ProcType     = (EaxReg >> 12) & 0x3;
  Cpu -> m_ProcFamily   = (EaxReg >> 8 ) & 0xf;
  Cpu -> m_ProcModel    = (EaxReg >> 4 ) & 0xf;
  Cpu -> m_ProcStepping = (EaxReg)       & 0x7;
  Cpu -> m_CPU_Features = EdxReg;

  /**** Intel CPU ****/

  if ((strncmp(Cpu -> m_ProcVendorString, VENDOR_INTEL, 12)) == 0) {
    Cpu -> m_CPU_Type           = EaxReg;
    Cpu -> m_ProcBrandID        = EbxReg & 0xF;
    strcpy(Cpu -> m_ProcName, DXWCPU_GetIntelProcName(Cpu));

    if (Cpu -> m_ProcFamily >= 6) {
      Cpu -> m_CPU_Suffix[Cpu -> m_CPU_Suffixes++] = CPU_SUFF_INTEL_686;
      if (Cpu -> m_ProcModel >= 7)
        Cpu -> m_CPU_Suffix[Cpu -> m_CPU_Suffixes++] = CPU_SUFF_INTEL_P3;
    }
    if (Cpu -> m_ProcFamily >= 15) Cpu -> m_CPU_Suffix[Cpu -> m_CPU_Suffixes++] = CPU_SUFF_INTEL_P4;

    /**** AMD CPU ****/

  } else if ((strncmp(Cpu -> m_ProcVendorString, VENDOR_AMD, 12))==0) {
    if (DXWCPU_GetCPUID_EAX(0x80000000)) {
      Cpu -> m_CPU_ExtendedFeatures = DXWCPU_GetCPUID_EDX(0x80000001);
      DXWCPU_GetCPUIDStringAMD(0x80000002, Cpu -> m_ProcName);
      DXWCPU_GetCPUIDStringAMD(0x80000003, Cpu -> m_ProcName + 16);
      DXWCPU_GetCPUIDStringAMD(0x80000004, Cpu -> m_ProcName + 32);
    } else Cpu -> m_CPU_ExtendedFeatures = EdxReg;

    if (Cpu -> m_ProcFamily >= 6)
      Cpu -> m_CPU_Suffix[Cpu -> m_CPU_Suffixes++] = CPU_SUFF_INTEL_686;
    if (Cpu -> m_ProcFamily >= 5)
      Cpu -> m_CPU_Suffix[Cpu -> m_CPU_Suffixes++] = CPU_SUFF_AMD_K6;
    if (Cpu -> m_ProcFamily >= 6) {
      Cpu -> m_CPU_Suffix[Cpu -> m_CPU_Suffixes++] = CPU_SUFF_AMD_K7;
      if (Cpu -> m_CPU_Features & INTEL_SSE_FLAG)
        Cpu -> m_CPU_Suffix[Cpu -> m_CPU_Suffixes++] = CPU_SUFF_AMD_K7SSE;
    }
    if (Cpu -> m_ProcFamily >= 15)
      Cpu -> m_CPU_Suffix[Cpu -> m_CPU_Suffixes++] = CPU_SUFF_AMD_A64;
  }

  /**** Set the suffixes ****/

  for(k = 0, j = Cpu -> m_CPU_Suffixes - 1; k < j; ++k, --j) {
    Tmp                    = Cpu -> m_CPU_Suffix[k];
    Cpu -> m_CPU_Suffix[k] = Cpu -> m_CPU_Suffix[j];
    Cpu -> m_CPU_Suffix[j] = Tmp;
  }

  return true;
}


/**** Obtain the Intel's processor name ****/

char *DXWCPU_GetIntelProcName(DXWCPU *Cpu)
{
  char          *CPUName = PROCESSOR_UNKNOWN;

  switch (Cpu -> m_ProcFamily) {

  case 5:       // 586 family
    if (Cpu -> m_ProcType == 1) CPUName = INTEL_PO;
    else CPUName = INTEL_P;
    break;

  case 6:       // 686 family
    switch(Cpu -> m_ProcModel) {
    case 1:
      CPUName = INTEL_PPRO;
      break;

    case 3:
      if (Cpu -> m_ProcType == 1) CPUName = INTEL_PIIO;
      else CPUName = INTEL_PII;
      break;

    case 5:
      CPUName = INTEL_PII;
      break;

    case 6:
      CPUName = INTEL_CELERON;
      break;

    case 7:
    case 8:
      CPUName = INTEL_PIII;
      break;

    case 10:
      CPUName = INTEL_XEON;
      break;

    case 0x0B:
      CPUName = INTEL_PIIIM;
      break;
    }
    break;

  case 15:      // Pentium 4 family
    Cpu -> m_ProcExtendedFamily = (Cpu -> m_CPU_Type >> 20) & 0xff;
    switch(Cpu -> m_ProcExtendedFamily) {
    case 0:     // Family = 15, Ext. Family = 0:  Pentium 4 (80786 ??) processor family
      Cpu -> m_ProcExtendedModel = (Cpu -> m_CPU_Type >> 16) & 0xFF;
      switch(Cpu -> m_ProcModel) {
      case 0:
      case 1:
        if (Cpu -> m_ProcBrandID == 8) CPUName = INTEL_PIV;
        else CPUName = INTEL_PIVXEON;
        break;

      case 2:
        if ((Cpu -> m_ProcBrandID == 9) || (Cpu -> m_ProcBrandID == 0xA)) CPUName = INTEL_PIV;
        else CPUName = INTEL_PIVXEON;
        break;

      default:
        CPUName = INTEL_PIV;
      } /* End of switch */
      break;

    case 1:     // Family = 15, Ext. Family = 1:  McKinley (64-bit) processor family
      CPUName = INTEL_MCKINLEY;
      break;

    default:
      CPUName = "Unknown Pentium 4+";
      break;
   } /* End of switch */
    break;
  } /* End of switch */

  return CPUName;
}


/**** Load library ****/

HINSTANCE DXWCPU_LoadLibrary(DXWCPU *Cpu, LPCTSTR lpszLibrary)
{
  char          Buf[MAX_PATH];
  char          *Prefix, **Suffix;

  HINSTANCE     hLib = NULL;

  strcpy(Buf, lpszLibrary);
  Prefix = Buf + strlen(Buf) - 4;
  for(Suffix = Cpu -> m_CPU_Suffix; (hLib == NULL) && (*Suffix); ++Suffix) {
    *Prefix = 0;
    strcat(Buf, *Suffix);
    strcat(Buf, ".dll");
    hLib = LoadLibrary(Buf);
  } /* End of for */

  return hLib;
}

