
/*********************************************************************
****              ESCHER Next Generation - Utilities              ****
**** (c) 1997-2003, G. Ausiello, G. Cesareni, M. Helmer Citterich ****
****               NG version by Alessandro Pedretti              ****
*********************************************************************/


#include <ctype.h>

#ifdef __WIN32__
#  define  DXWCPU_C
#  include <dxwcpu.h>

extern DXWCPU           EsCpu;
#endif

#include "my_macros.h"
#include "globstr.h"
#include "utils.h"

#  ifdef WIN32
#    define  strcasecmp         stricmp
#  endif


/**** Allocate the memory with error check ****/

void *Alloca(ES_ULONG Mem)
{
  void		*Ind;

  if ((Ind = (void *)ALLOC(Mem)) == NULL)
    fprintf(stderr, "Out of memory\n");

  return Ind;
}


/**** The file exists ****/

ES_BOOL FileExists(char *File)
{
  FILE          *FH;

  if ((FH = fopen(File, "r")) != NULL) {
    fclose(FH);
    return TRUE;
  }

  return FALSE;
}


/**** Return the file name ****/

char *GetFileName(char *Str)
{
  register char *Ptr = Str + strlen(Str);

  while ((Ptr != Str) &&
#ifdef AMIGA
         (*Ptr != ':') &&
#endif

#ifdef WIN32
         (*Ptr != '\\')
#else
         (*Ptr != '/')
#endif
        ) --Ptr;

  if (Ptr != Str) ++Ptr;

  return Ptr;
}


/**** Return the file name ****/

void GetFilePath(char *Path, char *Str)
{
  register char *Ptr;

  strcpy(Path, Str);
  Ptr = Path + strlen(Path);

  while ((Ptr != Path) &&
#ifdef AMIGA
         (*Ptr != ':') &&
#endif

#ifdef WIN32
         (*Ptr != '\\')
#else
         (*Ptr != '/')
#endif
        ) --Ptr;

  *Ptr = 0;
}


/**** Return the program path ****/

ES_BOOL GetPrgPath(char *Str, ES_BOOL DataDir)
{
  char          *CharPtr;

  if (!getenv("VEGADIR")) {
#ifdef AMIGA
    strcpy(Str, "PROGDIR:");
#else
#  ifdef WIN32
   if (GetModuleFileName(NULL, Str, MAX_PATH)) {
     for(CharPtr = Str + strlen(Str) - 1; *CharPtr; --CharPtr) {
       if (*CharPtr == '\\') {
         CharPtr[1] = 0;
         break;
       }
     } /* End of for */
   }
#  else
    return CatErr(MSG_ERR_UTILS_VEGADIR);
#  endif
#endif
  } else strcpy(Str, (char *)getenv("VEGADIR"));

  CharPtr = Str + strlen(Str) - 1;

#ifndef WIN32
  if ((*CharPtr != ':') && (*CharPtr != '/')) strcat(Str, "/");
#else
  if (*CharPtr != '\\') strcat(Str, "\\");
#endif

  /**** If requested, adds the Data subdirectory ****/

  if (DataDir)
#ifdef WIN32
    strcat(Str, "Data\\");
#else
    strcat(Str, "Data/");
#endif

  return TRUE;
}


/**** Load the preferences ****/

void LoadPrefs(ES_PREFS *Prf, char *FileName)
{
  char                  Buf[256], *Ptr;
  FILE                  *FH;

  static const char     *Sep = " \t\012\015";

  if ((FH = fopen(FileName, "r")) != NULL) {
    while(fgets(Buf, 255, FH)) {
      if (((Ptr = strtok(Buf, Sep)) != NULL) &&
          (*Ptr != ';')) {
        if ((!strcasecmp(Ptr, "LANGUAGE")) &&
            ((Ptr = strtok(NULL, Sep)) != NULL) &&
            (strlen(Ptr) < ES_PRF_LANGLEN)) {
          strcpy(Prf -> Language, Ptr);
          Lower(Prf -> Language);
        }
      }
    } /* End of while */
    fclose(FH);
  }
}


/**** Convert to lower case ****/

void Lower(register char *Str)
{
  do {
    if ((*Str >= 'A') && (*Str <= 'Z')) *Str += (char)32;
  } while (*++Str);
}



/**** Translates a string into a QCHAR ****/

void Str2Qchar(ES_QCHAR *Res, register char *Str)
{
  register char      *Dest = (*Res).C;
  register ES_UWORD  k;

  (*Res).L = (ES_ULONG)NULL;
  for(k = 0;(*Str) && (k < 4); ++k) {
   *Dest = *Str;
   ++Str;
   ++Dest;
  }
}


/**** Skip file lines ****/

ES_BOOL SkipLines(FILE *FH, char *Buf, ES_ULONG Lines)
{
  while(Lines > 0) {
    if (!fgets(Buf, 255, FH)) {
      PrintDosErr();
      return FALSE;
    }
   --Lines;
  } /* End of while */

  return TRUE;
}


/**** Change a string to uppercase ****/

void Upper(register char *Str)
{
  do *Str = (char)toupper(*Str); while(*++Str);
}

/***********************************
**** Windows specific functions ****
***********************************/

#ifdef WIN32

/**** Load the DLL function addresses ****/

ES_BOOL GetProcAddresses(HINSTANCE *hLibrary, LPCSTR lpszLibrary,
                        INT nCount, ... )
{
  va_list       va;

  if (*hLibrary == NULL) {
    va_start(va, nCount);
    if ((*hLibrary = DXWCPU_LoadLibrary(&EsCpu, lpszLibrary)) != NULL) {
      FARPROC   *lpfProcFunction = NULL;
      LPSTR     lpszFuncName     = NULL;
      INT       nIdxCount        = 0;
      while(nIdxCount < nCount) {
        lpfProcFunction = va_arg(va, FARPROC*);
        lpszFuncName = va_arg(va, LPSTR);
        if ((*lpfProcFunction = GetProcAddress(*hLibrary, lpszFuncName)) == NULL) {
          lpfProcFunction = NULL;
          FreeLibrary(*hLibrary);
          *hLibrary = NULL;
          CatErr(MSG_ERR_WIN_CANTGETADDR, lpszFuncName, lpszLibrary);
          return FALSE;
        }
        nIdxCount++;
      } /* End of while */
    } else {
      va_end(va);
      CatErr(MSG_ERR_WIN_CANTLOADLIB, lpszLibrary);
      return FALSE;
    }
    va_end(va);
  }

  return TRUE;
}

#endif

